@@ -50,24 +50,25 @@ logger = logging.getLogger(__name__)
 # https://github.com/open-contracting/lib-cove-ocds/blob/master/libcoveocds/common_checks.py
 
 
-def unique_ids(validator, ui, instance, schema, id_name="id"):
+def unique_ids(validator, ui, instance, schema, id_names=["id"]):
     if ui and validator.is_type(instance, "array"):
         non_unique_ids = set()
         all_ids = set()
         for item in instance:
             try:
-                item_id = item.get(id_name)
+                item_ids = tuple(item.get(id_name) for id_name in id_names)
             except AttributeError:
                 # if item is not a dict
-                item_id = None
-            if (
-                item_id
+                item_ids = None
+            if item_ids and all(
+                item_id is not None
                 and not isinstance(item_id, list)
                 and not isinstance(item_id, dict)
+                for item_id in item_ids
             ):
-                if item_id in all_ids:
-                    non_unique_ids.add(item_id)
-                all_ids.add(item_id)
+                if item_ids in all_ids:
+                    non_unique_ids.add(item_ids)
+                all_ids.add(item_ids)
             else:
                 if not uniq(instance):
                     msg = "Array has non-unique elements"
@@ -77,9 +78,12 @@ def unique_ids(validator, ui, instance, schema, id_name="id"):
                     return
 
         for non_unique_id in sorted(non_unique_ids):
-            msg = "Non-unique {} values".format(id_name)
-            err = ValidationError(msg, instance=non_unique_id)
-            err.error_id = "uniqueItems_with_{}".format(id_name)
+            if len(id_names) == 1:
+                msg = "Non-unique {} values".format(id_names[0])
+            else:
+                msg = "Non-unique combination of {} values".format(", ".join(id_names))
+            err = ValidationError(msg, instance=", ".join(non_unique_id))
+            err.error_id = "uniqueItems_with_{}".format("__".join(id_names))
             yield err
 
 
@@ -849,7 +853,7 @@ def get_schema_validation_errors(
         ]:
             instance = e.instance
 
-        if e.validator == "format" and e.validator not in ["date-time", "uri"]:
+        if e.validator == "format" and validator_type not in ["date-time", "uri"]:
             instance = e.instance
 
         if getattr(e, "error_id", None) in ["oneOf_any", "oneOf_each"]:
@@ -888,7 +892,9 @@ def get_schema_validation_errors(
         )
         if instance is not None:
             unique_validator_key["instance"] = instance
-        validation_errors[json.dumps(unique_validator_key)].append(value)
+        validation_errors[
+            json.dumps(unique_validator_key, default=decimal_default)
+        ].append(value)
     return dict(validation_errors)
 
 
